﻿using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.SimulationCore {
  [StorableClass]
  public sealed class Event<TModel> : IEvent<TModel> where TModel : IModel {
    private double time;
    public double Time { get { return time; } }

    public IAction<TModel> Action { get; private set; }

    #region Storable Properties
    // ReSharper disable UnusedMember.Local
    // ReSharper disable UnusedParameter.Local
    [Storable]
    private double StorableTime {
      get { return time; }
      set { time = value; }
    }
    [Storable]
    private IAction<TModel> StorableActions {
      get { return Action; }
      set { Action = value; }
    }
    [StorableConstructor]
    private Event(bool deserializing) { }
    // ReSharper restore UnusedParameter.Local
    // ReSharper restore UnusedMember.Local
    #endregion

    private Event(Event<TModel> original, Cloner cloner) {
      cloner.RegisterClonedObject(original, this);
      time = original.time;
      Action = cloner.Clone(original.Action);
    }
    public Event(double time) {
      this.time = time;
    }
    public Event(double time, IAction<TModel> action) {
      this.time = time;
      this.Action = action;
    }

    public object Clone() { return Clone(new Cloner()); }
    public IDeepCloneable Clone(Cloner cloner) {
      return new Event<TModel>(this, cloner);
    }
  }
}
